@@ -24,7 +24,7 @@ except ImportError:#pragma: no cover
 
 from webob import Request
 # TODO: Stop using Paste; we already started using WebOb
-from paste.httpexceptions import HTTPFound, HTTPUnauthorized
+from webob.exc import HTTPFound, HTTPUnauthorized
 from paste.request import construct_url, parse_dict_querystring, parse_formvars
 from zope.interface import implements
 
@@ -35,36 +35,36 @@ __all__ = ['FriendlyFormPlugin']
 
 class FriendlyFormPlugin(object):
     """
-    :class:`RedirectingFormPlugin 
+    :class:`RedirectingFormPlugin
     <repoze.who.plugins.form.RedirectingFormPlugin>`-like form plugin with
     more features.
-    
+
     It is like ``RedirectingFormPlugin``, but provides us with the following
     features:
-    
+
     * Users are not challenged on logout, unless the referrer URL is a
       private one (but that's up to the application).
     * Developers may define post-login and/or post-logout pages.
     * In the login URL, the amount of failed logins is available in the
-      environ. It's also increased by one on every login try. This counter 
+      environ. It's also increased by one on every login try. This counter
       will allow developers not using a post-login page to handle logins that
       fail/succeed.
-    
+
     You should keep in mind that if you're using a post-login or a post-logout
     page, that page will receive the referrer URL as a query string variable
     whose name is "came_from".
-    
+
     Forms can be submitted with any encoding (non-ASCII credentials are
     supported) and ISO-8859-1 (aka "Latin-1") is the default one.
-    
+
     """
     implements(IChallenger, IIdentifier)
-    
+
     def __init__(self, login_form_url, login_handler_path, post_login_url,
                  logout_handler_path, post_logout_url, rememberer_name,
                  login_counter_name=None, charset="iso-8859-1"):
         """
-        
+
         :param login_form_url: The URL/path where the login form is located.
         :type login_form_url: str
         :param login_handler_path: The URL/path where the login form is
@@ -87,13 +87,13 @@ class FriendlyFormPlugin(object):
         :param charset: The character encoding to be assumed when the user
             agent does not submit the form with an explicit charset.
         :type charset: :class:`str`
-        
+
         The login counter variable's name will be set to ``__logins`` if
         ``login_counter_name`` equals None.
-        
+
         .. versionchanged:: 1.0.1
             Added the ``charset`` argument.
-        
+
         """
         self.login_form_url = login_form_url
         self.login_handler_path = login_handler_path
@@ -105,21 +105,21 @@ class FriendlyFormPlugin(object):
         if not login_counter_name:
             self.login_counter_name = '__logins'
         self.charset = charset
-    
+
     # IIdentifier
     def identify(self, environ):
         """
         Override the parent's identifier to introduce a login counter
         (possibly along with a post-login page) and load the login counter into
         the ``environ``.
-        
+
         """
         request = Request(environ, charset=self.charset)
-        
+
         path_info = environ['PATH_INFO']
         script_name = environ.get('SCRIPT_NAME') or '/'
         query = request.GET
-        
+
         if path_info == self.login_handler_path:
             ## We are on the URL where repoze.who processes authentication. ##
             # Let's append the login counter to the query string of the
@@ -130,15 +130,21 @@ class FriendlyFormPlugin(object):
             try:
                 login = form['login']
                 password = form['password']
+            except KeyError:
+                credentials = None
+            else:
                 credentials = {
                     'login': login,
                     'password': password,
                     }
-            except KeyError:
-                credentials = None
+
+            if request.charset == "us-ascii":
+                credentials['login'] = str(credentials['login'])
+                credentials['password'] = str(credentials['password'])
+
             referer = environ.get('HTTP_REFERER', script_name)
             destination = form.get('came_from', referer)
-            
+
             if self.post_login_url:
                 # There's a post-login page, so we have to replace the
                 # destination with it.
@@ -152,7 +158,7 @@ class FriendlyFormPlugin(object):
                                                            query['came_from'])
             failed_logins = self._get_logins(environ, True)
             new_dest = self._set_logins_in_url(destination, failed_logins)
-            environ['repoze.who.application'] = HTTPFound(new_dest)
+            environ['repoze.who.application'] = HTTPFound(location=new_dest)
             return credentials
 
         elif path_info == self.logout_handler_path:
@@ -165,7 +171,7 @@ class FriendlyFormPlugin(object):
             environ['came_from'] = came_from
             environ['repoze.who.application'] = HTTPUnauthorized()
             return None
-            
+
         elif path_info == self.login_form_url or self._get_logins(environ):
             ##  We are on the URL that displays the from OR any other page  ##
             ##   where the login counter is included in the query string.   ##
@@ -177,14 +183,14 @@ class FriendlyFormPlugin(object):
             if self.login_counter_name in query:
                 del query[self.login_counter_name]
                 environ['QUERY_STRING'] = urlencode(query, doseq=True)
-    
+
     # IChallenger
     def challenge(self, environ, status, app_headers, forget_headers):
         """
         Override the parent's challenge to avoid challenging the user on
-        logout, introduce a post-logout page and/or pass the login counter 
+        logout, introduce a post-logout page and/or pass the login counter
         to the login form.
-        
+
         """
         url_parts = list(urlparse(self.login_form_url))
         query = url_parts[4]
@@ -198,7 +204,7 @@ class FriendlyFormPlugin(object):
         # Configuring the headers to be set:
         cookies = [(h,v) for (h,v) in app_headers if h.lower() == 'set-cookie']
         headers = forget_headers + cookies
-        
+
         if environ['PATH_INFO'] == self.logout_handler_path:
             # Let's log the user out without challenging.
             came_from = environ.get('came_from')
@@ -213,7 +219,7 @@ class FriendlyFormPlugin(object):
                 # Redirect to the referrer URL.
                 script_name = environ.get('SCRIPT_NAME', '')
                 destination = came_from or script_name or '/'
-        
+
         elif 'repoze.who.logins' in environ:
             # Login failed! Let's redirect to the login form and include
             # the login counter in the query string
@@ -221,7 +227,7 @@ class FriendlyFormPlugin(object):
             # Re-building the URL:
             destination = self._set_logins_in_url(destination,
                                                   environ['repoze.who.logins'])
-        
+
         return HTTPFound(destination, headers=headers)
 
     # IIdentifier
@@ -233,30 +239,30 @@ class FriendlyFormPlugin(object):
     def forget(self, environ, identity):
         rememberer = self._get_rememberer(environ)
         return rememberer.forget(environ, identity)
-    
+
     def _get_rememberer(self, environ):
         rememberer = environ['repoze.who.plugins'][self.rememberer_name]
         return rememberer
-    
+
     def _get_full_path(self, path, environ):
         """
         Return the full path to ``path`` by prepending the SCRIPT_NAME.
-        
+
         If ``path`` is a URL, do nothing.
-        
+
         """
         if path.startswith('/'):
             path = environ.get('SCRIPT_NAME', '') + path
         return path
-    
+
     def _get_logins(self, environ, force_typecast=False):
         """
         Return the login counter from the query string in the ``environ``.
-        
-        If it's not possible to convert it into an integer and  
-        ``force_typecast`` is ``True``, it will be set to zero (int(0)). 
+
+        If it's not possible to convert it into an integer and
+        ``force_typecast`` is ``True``, it will be set to zero (int(0)).
         Otherwise, it will be ``None`` or an string.
-        
+
         """
         variables = parse_dict_querystring(environ)
         failed_logins = variables.get(self.login_counter_name)
@@ -266,26 +272,26 @@ class FriendlyFormPlugin(object):
             except (ValueError, TypeError):
                 failed_logins = 0
         return failed_logins
-    
+
     def _set_logins_in_url(self, url, logins):
         """
         Insert the login counter variable with the ``logins`` value into
         ``url`` and return the new URL.
-        
+
         """
         return self._insert_qs_variable(url, self.login_counter_name, logins)
-    
+
     def _insert_qs_variable(self, url, var_name, var_value):
         """
         Insert the variable ``var_name`` with value ``var_value`` in the query
         string of ``url`` and return the new URL.
-        
+
         """
         url_parts = list(urlparse(url))
         query_parts = parse_qs(url_parts[4])
         query_parts[var_name] = var_value
         url_parts[4] = urlencode(query_parts, doseq=True)
         return urlunparse(url_parts)
-    
+
     def __repr__(self):
         return '<%s %s>' % (self.__class__.__name__, id(self))
